import { Component, OnInit } from "@angular/core";
import { Router } from "@angular/router";
import { NgbModal } from "@ng-bootstrap/ng-bootstrap";
import { BlockUI, NgBlockUI } from "ng-block-ui";
import { TableApiService } from "src/app/_services/table-api.service";
import { BuildingService } from "../../service/monitoring-api.service";
import { AddEditMasterComponent } from "../add-edit-master/add-edit-master.component";
import { ToastrService } from "ngx-toastr";
import { LoginService } from "../../service/login.service";

@Component({
  selector: "app-master-category",
  templateUrl: "./master-category.component.html",
  styleUrls: ["./master-category.component.css"],
})
export class MasterCategoryComponent implements OnInit {
  data: any;
  filteredRows: any[];
  searchTerm: string = "";
  rows: any = [];
  public breadcrumb: any;
  dataMasterCategori: any;

  @BlockUI("zeroConfiguration") blockUIZeroConfiguration: NgBlockUI;

  constructor(
    private tableApiservice: TableApiService,
    private modalService: NgbModal,
    private router: Router,
    private monitoringApiService: BuildingService,
    private toastr: ToastrService,
    private authService: LoginService,
  ) {}

  ngOnInit() {
    this.authService.checkTokenAndRedirect();
    this.breadcrumb = {
      mainlabel: "Master Category",
      links: [
        {
          name: "Home",
          isLink: false,
        },
        {
          name: "Master Category",
          isLink: false,
        },
      ],
    };
    this.fetchData();
  }

  fetchData() {
    this.monitoringApiService.getMasterData().subscribe((res) => {
      this.data = res.results.data;
      this.dataMasterCategori = res.results.data.find(
        (item) => item.name === "master_category"
      );
      // this.filteredRows = this.dataMasterCategori.headerDetailParam;
      this.filteredRows = this.dataMasterCategori.headerDetailParam.sort((a, b) => b.id - a.id);
    });
  }

  filterRows() {
    if (!this.searchTerm) {
      this.filteredRows = [
        ...this.data.find((item) => item.name === "master_category")
          .headerDetailParam,
      ];
    } else {
      this.filteredRows = this.data
        .find((item) => item.name === "master_category")
        .headerDetailParam.filter((row) => this.rowContainsSearchTerm(row));
    }
  }

  rowContainsSearchTerm(row: any): boolean {
    const searchTermLC = this.searchTerm.toLowerCase();
    console.log(searchTermLC);
    
    return Object.values(row).some(
      (value) =>
        value !== null && value.toString().toLowerCase().includes(searchTermLC)
    );
  }

  openAddMasterModal() {
    const modalRef = this.modalService.open(AddEditMasterComponent, {
      size: "lg",
      backdrop: 'static',  // Add this line
      keyboard: false  // Add this line
    });

    modalRef.componentInstance.headerId = this.dataMasterCategori.id;
    modalRef.componentInstance.mode = "add";
    modalRef.componentInstance.category = true;
    modalRef.result.then(
      (result) => {
        console.log(result);
        if (result) {
          if (this.filteredRows.some(value => value.name.toLowerCase() === result.name.toLowerCase())) {
            this.toastr.error("Warning", "Data yang anda masukan double.", {
              timeOut: 5000,
              closeButton: true,
            });
          } else {
            this.monitoringApiService
            .postHeaderDetailParam(result)
            .subscribe((res) => {
              console.log(res);
              this.fetchData();
            });
          }
        }

        // if (result) {
        //   this.monitoringApiService.postHeaderDetailParam(result).subscribe(
        //     (res) => {
        //       this.fetchData();
        //       this.toastr.success("Success", "Save Completed.", {
        //         timeOut: 2000,
        //         closeButton: true,
        //       });
        //     },
        //     (error) => {
        //       console.error(error);
        //       this.toastr.error("Error", "Something went wrong!", {
        //         timeOut: 2000,
        //         closeButton: true,
        //       });
        //     }
        //   );
        // }
      },
      (reason) => {
        console.log(`Dismissed: ${reason}`);
      }
    );
  }

  editRow(row) {
    const modalRef = this.modalService.open(AddEditMasterComponent, {
      size: "lg",
      backdrop: 'static',  // Add this line
      keyboard: false  // Add this line
    });

    modalRef.componentInstance.headerId = this.dataMasterCategori.id;
    modalRef.componentInstance.dataRow = row;
    modalRef.componentInstance.mode = "edit";
    modalRef.componentInstance.category = true;
    modalRef.result.then(
      (result) => {
        console.log(result);
        if (result) {
          if (this.filteredRows.some(value => value.name.toLowerCase() === result.name.toLowerCase() && value.status === result.status)) {
            this.toastr.error("Warning", "Data yang anda masukan double.", {
              timeOut: 5000,
              closeButton: true,
            });
          } else {
            this.monitoringApiService
            .putHeaderDetailParam(result, row.id)
            .subscribe((res) => {
              console.log(res);
              this.fetchData();
            });
          }
        }
        // if (result) {
        //   this.monitoringApiService
        //     .putHeaderDetailParam(result, row.id)
        //     .subscribe(
        //       (res) => {
        //         this.fetchData();
        //         this.toastr.success("Success", "Update Completed.", {
        //           timeOut: 2000,
        //           closeButton: true,
        //         });
        //       },
        //       (error) => {
        //         console.error(error);
        //         this.toastr.error("Error", "Something went wrong!", {
        //           timeOut: 2000,
        //           closeButton: true,
        //         });
        //       }
        //     );
        // }
      },
      (reason) => {
        console.log(`Dismissed: ${reason}`);
      }
    );
  }

  deleteRow(row) {
    const confirmDelete = confirm("Are you sure you want to delete this item?");
    if (confirmDelete) {
      this.monitoringApiService.deleteHeaderDetailParam(row.id).subscribe(
        (res) => {
          this.fetchData();
          this.toastr.success("Success", "Delete Completed.", {
            timeOut: 2000,
            closeButton: true,
          });
        },
        (error) => {
          console.error(error);
          this.toastr.error("Error", "Something went wrong!", {
            timeOut: 2000,
            closeButton: true,
          });
        }
      );
    }
  }
}
