import { Component, ViewChild } from "@angular/core";
import { FormBuilder, FormGroup, NgForm, Validators } from "@angular/forms";
import { LoginService } from "../../service/login.service";
import { ToastrService } from "ngx-toastr";
import { jwtDecode } from "jwt-decode";
import { AuthService } from "src/app/_services/auth.service";
import { Router } from "@angular/router";
import * as _ from "lodash"
interface CustomJwtPayload {
  exp: number;
  scope: string;
  iat: number;
  preferred_username: string;
  name: string;
  email: string;
  family_name: string;
  given_name: string;
  sub: string;
}
@Component({
  selector: "app-profil-information",
  templateUrl: "./profil-information.component.html",
  styleUrls: ["./profil-information.component.css"],
})
export class ProfilInformationComponent {
  @ViewChild("f", { read: true }) userProfileForm: NgForm;
  public breadcrumb: any;
  profilInfo: FormGroup;
  activeTab: string = "profile-info2";
  submitted = false;
  storedData: any;
  currentUser: any;
  disableButton: boolean = false;
  oldData: any;
  url: any = "https://www.w3schools.com/howto/img_avatar.png";
  fileSelected: any = null;

  constructor(
    private formBuilder: FormBuilder,
    private authService: LoginService,
    private toastr: ToastrService,
    public logoutService: AuthService,
    private router: Router
  ) {
    this.authService._tabSelected.subscribe(res => {
      console.log(res);
      console.log(this.oldData);
      console.log(this.profilInfo.value);
      if (res === "update-password") {
        if (this.oldData !== this.profilInfo.value) {
          console.log('ada perubahan');
          return false
        } else {
          console.log('tidak ada perubahan');
          
        }
      }
    })
  }

  ngOnInit(): void {
    this.storedData = JSON.parse(localStorage.getItem("account_info"));
    this.currentUser = JSON.parse(localStorage.getItem("currentUser"));
    this.authService.checkTokenAndRedirect();

    this.profilInfo = this.formBuilder.group({
      firstName: ["", Validators.required],
      lastName: ["", Validators.required],
      email: ["", [Validators.required, Validators.pattern(/^[a-zA-Z0-9._%+-]+@[a-zA-Z0-9.-]+\.[a-zA-Z]{2,}$/)]],
      phone: ["", [Validators.required, Validators.pattern(/^[0-9]{10,15}$/)]],
      image: [null, Validators.required],
    });
    this.dataProfil(this.storedData.sub);
    this.profilInfo.valueChanges.subscribe(() => {
      this.disableButton = !this.profilInfo.valid;
    });
  }

  dataProfil(userId) {
    this.authService.getDataProfil(userId).subscribe((data) => {
      this.profilInfo.patchValue({
        firstName: data.data.firstname,
        lastName: data.data.lastname,
        email: data.data.email,
        phone: data.data.phone,
        image: [null],
      });
      // this.oldData._.cloneDeep(this.profilInfo.value);
      this.oldData = _.cloneDeep(this.profilInfo.value)
      console.log(this.oldData);
      
      if (
        data.data.image_path !== "https://kapi.absys.ninja/hemat/image/null"
      ) {
        this.url = data.data.image_path;
      }
    });
  }

  onSelectFile(event) {
    if (event.target.files && event.target.files[0]) {
      const file = event.target.files[0];
      this.fileSelected = file;
      if (!file.type.startsWith("image")) {
        this.toastr.error("Warning", "Please select an image file.", {
          timeOut: 5000,
          closeButton: true,
        });
        this.profilInfo.patchValue({
          image: null,
        });
        return;
      }

      const reader = new FileReader();
      reader.readAsDataURL(file);

      reader.onload = (e) => {
        this.url = e.target.result;
      };
    }
  }

  public deleteImage() {
    this.url = null;
    if (this.url === null) {
      this.disableButton = true;
    } else {
      this.disableButton = false;
    }
    this.profilInfo.patchValue({ image: null });
  }

  selectTab(tabName: string) {
    this.activeTab = tabName;
  }

  get f() {
    return this.profilInfo.controls;
  }

  onProjectInfoSubmit() {
    this.submitted = true;
    if (this.profilInfo.invalid) {
      this.disableButton = true;
      return;
    }

    this.disableButton = false;

    const formData = new FormData();
    formData.append("firstname", this.profilInfo.get("firstName").value);
    formData.append("lastname", this.profilInfo.get("lastName").value);
    formData.append("email", this.profilInfo.get("email").value);
    formData.append("phone", this.profilInfo.get("phone").value);
    formData.append("statusId", "2");
    formData.append("buildingId", "4");
    formData.append("image", this.fileSelected);

    this.authService.updateProfile(formData, this.storedData.sub).subscribe(
      (response) => {
        console.log("Profile updated successfully:", response);
        this.authService
          .updateUserProfile(this.currentUser.refresh_token)
          .subscribe((resp) => {
            const decodedToken = jwtDecode<CustomJwtPayload>(resp.access_token);
            localStorage.setItem("account_info", JSON.stringify(decodedToken));
            const userProfile = {
              access_token: resp.access_token,
              refresh_token: resp.refresh_token,
              displayName: decodedToken.name,
              buildingId: 4,
            };
            localStorage.setItem("currentUser", JSON.stringify(userProfile));
            window.location.reload();
            this.toastr.success("success", "Profile updated successfully.", {
              timeOut: 5000,
              closeButton: true,
            });
          });
      },
      (error) => {
        console.error("Profile update failed:", error);
        this.toastr.error("Error", "Profile update failed.", {
          timeOut: 5000,
          closeButton: true,
        });
        // Handle error response
      }
    );
  }

  cancel() {
    this.dataProfil(this.storedData.sub);
    this.disableButton = false;
  }
}
